#ifndef __monarch_h__
#define __monarch_h__

#include <stdio.h>
#include <netinet/tcp.h>
#include <sys/time.h>

#include "common.h"

class ProbeTrace;
class KernelTrace;

class Sender {
protected:
  struct sockaddr_in localAddress;
  struct sockaddr_in remoteAddress;
  const char *messageFileName;
  KernelTrace *kernelTrace;
  int writeGranularity;
  int sendBufferSize;
  int transferSize;
  struct timeval transferEnd;
  
public:
  Sender(struct sockaddr_in *localAddress, struct sockaddr_in *remoteAddress);
  void setMessageFile(const char *messageFileName) { this->messageFileName = messageFileName; };
  void setWriteGranularity(int granularityBytes) { if (granularityBytes > 0) this->writeGranularity = granularityBytes; };
  void setSendBufferSize(int bufferSizeBytes) { if (bufferSizeBytes > 0) this->sendBufferSize = bufferSizeBytes; };
  void setTransferSize(int bytes) { if (bytes > 0) this->transferSize = bytes; };
  void setTransferTime(int seconds) { if (seconds > 0){ struct timeval tv; gettimeofday(&tv, NULL); this->transferEnd.tv_sec = tv.tv_sec + seconds; this->transferEnd.tv_usec = tv.tv_usec; } verbose(1, "Monarch will run for %d seconds",seconds);  };
  void setKernelTrace(KernelTrace *trace) { this->kernelTrace = trace; };
  void run();
};

class Receiver {
protected:
  struct sockaddr_in localAddress;
  struct sockaddr_in remoteAddress;
  KernelTrace *kernelTrace;
  int receiveBufferSize;
  int readGranularity;
  int myServerSocket;
  bool initialized;
  char *buffer;
  
public:
  Receiver(struct sockaddr_in *localAddress, struct sockaddr_in *remoteAddress);
  void setReceiveBufferSize(int bufferSizeBytes) { if (bufferSizeBytes > 0) this->receiveBufferSize = bufferSizeBytes; };
  void setReadGranularity(int granularityBytes) { if (granularityBytes > 0) this->readGranularity = granularityBytes; };
  void setKernelTrace(KernelTrace *trace) { this->kernelTrace = trace; };
  void init();
  void run();
};

class Diagnostics {
protected:
  const static int FEATURE_CROSSTRAFFIC          = 1<<0;
  const static int FEATURE_LOSSES_DOWNSTREAM     = 1<<1;
  const static int FEATURE_LOSSES_UPSTREAM       = 1<<2;
  const static int FEATURE_REORDERING_DOWNSTREAM = 1<<3;
  const static int FEATURE_REORDERING_UPSTREAM   = 1<<4;
  const static int FEATURE_REORDERING_MESS       = 1<<5;
  const static int FEATURE_UNKNOWN               = 1<<6;

  int probesSent;
  int responsesReceived;
  int *receiveSequence;
  int *ipidSequence;
  
  bool ipidSanityCheck(char *buf);
  void findPermutation(int *seqA, int *seqB, int *perm, int len);
  void condense(int *a, int aLen);
  int levenshteinDistance(int *a, int aLen, int *b, int bLen);
  
public:
  Diagnostics(ProbeTrace *probeTrace);
  void write(FILE *stream);
};

#endif /* defined(__monarch_h__) */
