#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "probes.h"

Hashtable::Hashtable()
{
  for (int i=0; i<maxEntries; i++)
    entry[i].key = -1;
}

void Hashtable::put(int key, void *value)
{
  int idx = (key % maxEntries);
  int trials = 0;
  while (entry[idx].key != -1) {
    idx = (idx+1) % maxEntries;
    if ((++trials) == maxEntries)
      panic(ERR_INTERNAL, "Overflow in Hashtable; increase maxEntries!");
  }
  
  assert((0<=idx) && (idx<maxEntries) && (entry[idx].key == -1));
  entry[idx].key = key;
  entry[idx].value = value;
}

void *Hashtable::get(int key)
{
  int idx = (key % maxEntries);
  int trials = 0;
  do {
    if (entry[idx].key == key)
      return entry[idx].value;
    if (entry[idx].key == -1)
      break;
    
    idx = (idx+1) % maxEntries;
  } while ((++trials) < maxEntries);
  
  return NULL;
}
