#include <stdlib.h>
#include <string.h>

#include "monarch.h"
#include "probes.h"
#include "proxies.h"

#define dprintf(a...) do { } while (0)
//#define dprintf(a...) printf(a)

IcmpEchoRequestPolicy::IcmpEchoRequestPolicy() : ProbePolicy()
{
  this->nextSequenceNo = 0;
  this->packetTable = new Hashtable();
}

void IcmpEchoRequestPolicy::receivePacketFromSender(Packet *packet)
{
  dprintf("Receive packet from sender\n");
  int thisSequenceNo = nextSequenceNo++;
  packetTable->put(thisSequenceNo, new Packet(packet));
  proxy->sendPacketToRemote(makeIcmpEchoRequest(packet, thisSequenceNo));
  probeTrace->probeSent(thisSequenceNo);
}

void IcmpEchoRequestPolicy::receivePacketFromRemote(Packet *packet)
{
  dprintf("Receive packet from remote\n");
  struct IpHeader *iph = (struct IpHeader*)packet->getBuffer();
  if (iph->ip_proto != IPPROTO_ICMP)
    return;
    
  uint8 *icmpHeader = &(packet->getBuffer()[packet->getIpHeaderLength()]);
  if ((icmpHeader[0] != 0) || (icmpHeader[1] != 0))
    return;

  struct IcmpEchoRequest *ier = (struct IcmpEchoRequest*) icmpHeader;
  int thisIndex = ntohs(ier->sequenceno);
  Packet *savedPacket = (Packet*) packetTable->get(thisIndex);
  
  if (!savedPacket)
    panic(ERR_INTERNAL, "EchoRequestPolicy: No saved packet found for incoming EchoReply #%d", thisIndex);
  
  probeTrace->responseReceived(thisIndex, packet->getIpId());
  proxy->forwardPacketToReceiver(savedPacket);
}

void IcmpEchoRequestPolicy::receivePacketFromReceiver(Packet *packet)
{
  dprintf("Receive packet from receiver\n");
  proxy->forwardPacketToSender(packet);
}

Packet *IcmpEchoRequestPolicy::makeIcmpEchoRequest(Packet *packet, int sequenceNo)
{
  Packet *ping = new Packet();
  
  struct IcmpEchoRequest *ier = (struct IcmpEchoRequest*) &(ping->getBuffer()[20]);
  struct IpHeader *ip = (struct IpHeader*) ping->getBuffer();
  uint16 iplen = packet->getIpLength();
  ping->setLength(iplen);

  memset(ping->getBuffer(), 0, ping->getBufferLen());
  ip->ip_proto = IPPROTO_ICMP; 
  ip->ip_ttl = 60; 
  ip->ip_tos = 0; 
  ip->ip_vhl = 0x45; 
  ip->ip_off = htons(IP_DF); 
  ip->ip_len = htons(iplen);
  ip->ip_id = 0;

  ier->type = 8;
  ier->code = 0;
  ier->checksum = 0;
  ier->identifier = ~sequenceNo;
  ier->sequenceno = htons(sequenceNo);
  ier->checksum = htons(Packet::checksum((uint8*)ier, iplen-20));

  return ping;
}
